rm(list=ls())
set.seed(123)

p11 <- 0.15
p10 <- 0.20
p00 <- 0.25
p01 <- 1 - (p11 + p10 + p00)
p01 #0.4

n <- 10^5

U <- runif(n)

#gives negative correlation
b <- 2
a <- 1/b

#gives positive correlation:
#b <- 0.2
#a <- 1/b

Z <- matrix(nrow=n, ncol=2)

Z[(U <= p11),1] <- a
Z[(U <= p11),2] <- a

Z[(p11 < U) & (U <= (p11 + p10)),1] <- b
Z[(p11 < U) & (U <= (p11 + p10)),2] <- -b

Z[((p11 + p10) < U) & (U <= (p11 + p10 + p00)),1] <- -a
Z[((p11 + p10) < U) & (U <= (p11 + p10 + p00)),2] <- -a

Z[(U >(p11 + p10 + p00) ),1] <- -b
Z[(U >(p11 + p10 + p00) ),2] <- b

plot(Z)
abline(v=0)
abline(h=0)


#Let's confirm that this is correct:

p11 #0.15
mean((Z[,1] == a) & (Z[,2] == a)) #0.15029

p10 #0.2
mean((Z[,1] == b) & (Z[,2] == -b)) # 0.19986

p00 #0.25
mean((Z[,1] == -a) & (Z[,2] == -a)) #  0.25118

p01 #0.4
mean((Z[,1] == -b) & (Z[,2] == b)) # 0.39867
#Looks good.

mean(Z[,1]*Z[,2])
a^2*(p11 + p00) - b^2*(p10 + p01)
#confirmed.

#P(Z_1 = a) = p11 = P(Z_2 = a)
mean(Z[,1] == a)
mean(Z[,2] == a)
p11
#confirmed.


mean(Z[,1])
a*p11 + b*p10 - a*p00 - b*p01
#confirmed


cov(Z[,1], Z[,2])
a^2*(p11 + p00 - (p11 - p00)^2) - b^2*(p10 + p01 - (p10 - p01)^2 )
#confirmed

#$c_1^2 = (p_{11} + p_{00} - (p_{11} - p_{00})^2)^2$, 
#and 
#$c_2^2 = (p_{10} + p_{01} - (p_{10} - p_{01})^2)^2$

c1 <- (p11 + p00 - (p11 - p00)^2)
c1 # 0.39

c2 = (p10 + p01 - (p10 - p01)^2)
c2 #0.56

#$\Delta = (p_{11} - p_{00})(p_{10} - p_{01})$
Delta <- (p11 - p00)*(p10 - p01)
Delta #0.02

#$d = (p_{11} + p_{00} - (p_{11} - p_{00})^2 ) (p_{10} p_{01} - (p_{10} - p_{01})^2) - 4 \Delta^2$
d = (p11 + p00 - (p11 - p00)^2 )* (p10*p01 - (p10 - p01)^2) - 4*Delta^2
d


#$q = a^2 (p_{11} + p_{00} - (p_{11} - p_{00})^2) + b^2 (p_{10} + p_{01} - (p_{10} - p_{01})^2 )$ 
q <-a^2*(p11 + p00 - (p11 - p00)^2) + b^2*(p10 + p01 - (p10 - p01)^2 )
q

var(Z[,1])
q-2*Delta
#match

var(Z[,2])
q+2*Delta
#match


#\Var(\latent_1) \Var(\latent_2) & = (q - 2 \Delta) (q + 2\Delta) ?
var(Z[,1])*var(Z[,2])
(q-2*Delta)*(q+2*Delta)
#match
q^2-4*Delta^2
#match
a^4*c1^2 + b^4*c2^2 + 2*a^2*b^2*c1*c2 - 4*Delta^2
#match

cov(Z[,1], Z[,2])
a^2*c1 - b^2*c2
#match


#\rho = \frac{a^2 c_1 - b^2 c_2}{\sqrt{a^4 c_1^2 + b^4 c_2^2 + 2 a^2 b^2 c_1 c_2 - 4 \Delta^2}}
cor(Z[,1], Z[,2])
(a^2*c1 - b^2*c2)/(sqrt(a^4*c1^2 + b^4*c2^2 + 2*a^2*b^2*c1*c2 - 4*Delta^2))
#match

d <- 2*c1*c2 - 4*Delta^2
(a^2*c1 - b^2*c2)/(sqrt(a^4*c1^2 + b^4*c2^2 + d))
#match